{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Simple sample player object
//  Piotr Kardasz pik33@o2.pl
//  Plays a 512 16-bit stereo samples buffer in endless loop
//  Used dither code from WAV player engine by Kwabena W. Agyeman
//  v.0.8 beta - 20120625
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}


  _clkmode = xtal1+pll16x
  _clkfreq = 80_000_000

  _SD_DO = 0
  _SD_CLK = 1
  _SD_DI = 2
  _SD_CS = 3
  _SD_WP = -1 ' -1 ifnot installed.
  _SD_CD = -1 ' -1 ifnot installed.

VAR

  long cog
  long buf[512]
  long bufnum



pub getbuf
return @buf

pub getbufnum
return bufnum


PUB start(left, right, ditherlevel)

'**** Starts a dithering sample player cog. Dither level=2..31

  longfill(@buf,0,512)
  bufptr:=@buf
  stop

'**** Kye's dither procedure initialization

  ditherLeftCounterSetup := ((left & $1F) + constant(%00110 << 26))
  ditherRightCounterSetup := ((right & $1F) + constant(%00110 << 26))
  ditherOutputMask := |<left | |<right
  leftDitherShift := rightDitherShift := (ditherlevel & $1F)
    
'**** start a cog

    cog:=1+cognew(@init, @bufnum)

    return cog
  
PUB stop

if cog>0
  cogstop(cog-1)
cog := 0
  

DAT

' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'                       DAC Dither Driver
' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

                        org     0

' //////////////////////Initialization/////////////////////////////////////////////////////////////////////////////////////////

init                    mov     ctra,               ditherLeftCounterSetup     ' Setup counter modes to duty cycle mode.
                        mov     ctrb,               ditherRightCounterSetup
                        mov     dira,               ditherOutputMask
  '
                        mov     bufptr2,par
                        mov     time,cnt
                        add     time,delay


' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'                       Dither
' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

loop                    cmp     time,cnt      wc   'if it is not time to get next sample
                 if_c   jmp     #p01
                        nop
                        call    #dither
                        jmp     #loop

p01

                        add     time,delay
                        call    #dither
                        mov     ptr,bufptr
                        add     ptr,bufcnt         'pointer to next sample
                        rdword  lsample,ptr        'get left sample
                        call    #dither            '16 instr
                        add     lsample,offset     'convert to unsigned
                        add     ptr, #2
                        rdword  rsample,ptr        'hub window after 18 instr.
                        call    #dither
                        and     lsample,ffff
                        shl     lsample,#13        'multiply *$E000 to give it margin for dither
                        mov     temp,lsample
                        shl     lsample,#1
                        call    #dither
                        add     temp,lsample
                        shl     lsample,#1
                        add     lsample,temp
                        add     lsample,offset2    '$10000000 up!
                        call    #dither
                        add     rsample,offset
                        and     rsample,ffff
                        shl     rsample,#13
                        mov     temp,rsample
                        call    #dither
                        shl     rsample,#1
                        add     temp,rsample
                        shl     rsample,#1
                        add     rsample,temp
                        call    #dither
                        add     rsample,offset2
                        mov     rs2,rsample
                        mov     ls2,lsample
                        add     bufcnt,#4
                        call    #dither
                        nop
                        and     bufcnt,bufmask
                        wrlong  bufcnt,bufptr2
                        call    #dither
                        jmp    #loop

' **** Kye's dither procedure

dither                  test    leftLFSR,           leftTaps wc                ' Iterate left dither source
                        rcl     leftLFSR,           #1                         '

                        test    rightLFSR,          rightTaps wc               ' Iterate right dither source
                        rcl     rightLFSR,          #1                         '

                        mov     ditherlbuffer,      ls2
                        mov     ditherLCounter,     leftLFSR                   '
                        sar     ditherLCounter,     leftDitherShift            '

                        mov     ditherrbuffer,      rs2
                        mov     ditherRCounter,     rightLFSR                  '
                        sar     ditherRCounter,     rightDitherShift           '

                        add     ditherLBuffer,      ditherLCounter             ' Apply dither.
                        mov     frqa,               ditherLBuffer              ' Output.

                        add     ditherRBuffer,      ditherRCounter             ' Apply dither.
                        mov     frqb,               ditherRBuffer              ' Output.
dither_ret              ret                                    '               ' 15 instruction
                        
' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'                       Data
' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



leftTaps                long    $A4_00_00_80                                   ' Left LFSR taps.
rightTaps               long    $80_A0_10_00                                   ' Right LFSR taps.

leftLFSR                long    1                                              ' Initial value.
rightLFSR               long    1                                              ' Initial value.

' //////////////////////Configuration Settings/////////////////////////////////////////////////////////////////////////////////

ditherLeftCounterSetup  long    0
ditherRightCounterSetup long    0
ditherOutputMask        long    0

leftDitherShift         long    0
rightDitherShift        long    0

' //////////////////////Addresses//////////////////////////////////////////////////////////////////////////////////////////////

ditherLeftAddress       long    0
ditherRightAddress      long    0

' //////////////////////Run Time Variables/////////////////////////////////////////////////////////////////////////////////////

time       long 0
delay      long 1812
lsample    long 0
rsample    long 0
ls2 long 0
rs2 long 0
bufmask    long %0000_0000_0000_0000_0000_0111_1111_1111
ptr        long 0
temp long 0

bufptr  long 0
bufcnt  long 0
bufptr2 long 0
offset2 long $1000_0100
offset  long $8000
ffff    long %1111_1111_1111_1111

ditherLBuffer           res     1
ditherLCounter          res     1
ditherRBuffer           res     1
ditherRCounter          res     1




' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

                        fit     496



{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}
